﻿--[[
InviteOMatic Ace2 version, for WoW 2.0

Desc: InviteOMatic is an invitemod for helping to invite your friends/guildmates/other to a group/raid.
Author: Gof (Tait of Aszune)

Revision:
3.0 --> 3.1
	Added Fubar2.0 support.
	Fixed a bug that prevented the channel list from showing up first time the addon was loaded.
	Updated libs.

2.06 --> 3.0
	Rewrote the code completely, and in the same round wrote it for the Ace2 framework
	Removed all BG inviting, as its now in the blizz ui. (1 big raid enforced)
	
TODO:
	Whisper inviting of others, ie: invite blah plz
]]--

InviteOMatic = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "FuBarPlugin-2.0")

InviteOMatic:RegisterDB("InviteOMaticDB", "InviteOMaticDBPC")
InviteOMatic:RegisterDefaults("profile", {
	watchedChannels = {
	},
	invitePatterns = {
		"invite",
	},
	ignoreList = {
	},
	dbg = false,
} )

InviteOMatic.options = { 
    type='group',
    args = {
		invite = {
			type='group',
			name="Invite",
			desc="Invite settings",
			args = {
				addWord = {
					type = 'text',
					name = "Add inviteword",
					desc = "Adds the specified inviteword",
					usage = "",
					set = "AddInviteWord",
					get = "GetAddedInvite",
				},
				deleteWord = {
					type='group',
					name="Delete inviteword",
					desc="Delete a inviteword",
					args = {},
				}
			}
		},
		channel = {
			type='group',
			name="Channel",
			desc="Channel watch settings",
			args = {
				addChannel = {
					type = 'text',
					name = "Add channel",
					desc = "Adds the channel to list of watched channels",
					usage = "",
					set = "AddChannel",
					get = "GetAddedChannel",
				},
				deleteChannel = {
					type='group',
					name="Delete Channel",
					desc="Delete this channel from channels watched",
					args = {},
				}
			}
		},
		ignore = {
			type='group',
			name="Ignore",
			desc="Ignore options",
			args = {
				add = {
					type = 'text',
					name = "Add to ignorelist",
					desc = "Adds the specified person to your ignorelist",
					usage = "",
					set = "AddToIgnore",
					get = "GetAddedIgnore",
				},
				delete = {
					type='group',
					name="Delete",
					desc="Delete a person from your ignorelist",
					args = {},
				}
			}
		},
		debug = {
            type = 'toggle',
            name = 'debug',
            desc = 'Toggle debug messages',
            usage = "",
            get = "IsDebug",
            set = "ToggleDebug",			
		}
	}
}

InviteOMatic.OnMenuRequest = InviteOMatic.options;

InviteOMatic:RegisterChatCommand({"/inviteomatic", "/iom"}, InviteOMatic.options)

local tablet = AceLibrary("Tablet-2.0")

function InviteOMatic:OnTooltipUpdate()
    local cat = tablet:AddCategory(
        'text', "InviteWords",
        'columns', 1,
        'child_textR', 1,
        'child_textG', 1,
        'child_textB', 0,
        'child_textR2', 1,
        'child_textG2', 1,
        'child_textB2', 1
    )
		
	local added = false;
	
	for key,value in pairs(self.db.profile.invitePatterns) do
		cat:AddLine('text', value)
		added = true;
    end
	
	if( not added ) then
		cat:AddLine('text', "none");
	end

    cat = tablet:AddCategory(
        'text', "Invite Channels",
        'columns', 1,
        'child_textR', 1,
        'child_textG', 1,
        'child_textB', 0,
        'child_textR2', 1,
        'child_textG2', 1,
        'child_textB2', 1
    )

	added = false;
    
	for key,value in pairs(self.db.profile.watchedChannels) do
		cat:AddLine('text', value)
		added = true;
    end

	if( not added ) then
		cat:AddLine('text', "none");
	end
    
    cat = tablet:AddCategory(
        'text', "Ignored people",
        'columns', 1,
        'child_textR', 1,
        'child_textG', 1,
        'child_textB', 0,
        'child_textR2', 1,
        'child_textG2', 1,
        'child_textB2', 1
    )
	
	added = false;
	
	for key,value in pairs(self.db.profile.ignoreList) do
		cat:AddLine('text', value)
		added = true;
    end

	if( not added ) then
		cat:AddLine('text', "none");
	end

    tablet:SetHint("Right-Click to open menu")
end
	
--[[
	local dewdrop = AceLibrary("Dewdrop-2.0")

dewdrop:Register(Minimap,
    'children', function()
        dewdrop:FeedAceOptionsTable(InviteOMatic.options)
    end
)
]]--

function InviteOMatic:AddToIgnore(name)
	if( string.len(name) == 0 ) then
		self:Debug("0-length ignores not allowed");
		return;
	end
	self:Debug(name.." is now on ignore");
	tinsert(self.db.profile.ignoreList, name);
	self:UpdateIgnoreList();
end

function InviteOMatic:AddInviteWord(name)
	if( string.len(name) == 0 ) then
		self:Debug("0-length invite words not allowed");
		return;
	end
	self:Debug(name.." is added as inviteword");
	tinsert(self.db.profile.invitePatterns, name);
	self:UpdatePatternsList();
end

function InviteOMatic:AddChannel(name)
	if( string.len(name) == 0 ) then
		self:Debug("0-length channel names not allowed");
		return;
	end
	self:Debug(name.." is added as a watch channel");
	tinsert(self.db.profile.watchedChannels, name);
	self:UpdateChannelList();
end

function InviteOMatic:GetAddedIgnore()
	--return self.db.profile.ignoreList[table.getn(self.db.profile.ignoreList)];
	return;
end

function InviteOMatic:GetAddedInvite()
	--return self.db.profile.invitePatterns[table.getn(self.db.profile.invitePatterns)];
	return;
end

function InviteOMatic:GetAddedChannel()
	--return self.db.profile.watchedChannels[table.getn(self.db.profile.watchedChannels)];
	return;
end

function InviteOMatic:ResetPatterns()
	self.db.profile.invitePatterns = {};
	self:UpdatePatternsList();
end

function InviteOMatic:UpdatePatternsList()
	self.options.args.invite.args.deleteWord.args = {};
	for key,value in pairs(self.db.profile.invitePatterns) do
		local arg = {
			type = 'execute',
			name = value,
			desc = "Delete this inviteword",
			func = function() InviteOMatic:DeleteInvitePattern(key); end,
		};
		tinsert(self.options.args.invite.args.deleteWord.args, arg);
	end
end

function InviteOMatic:UpdateChannelList()
	self.options.args.channel.args.deleteChannel.args = {};
	for key,value in pairs(self.db.profile.watchedChannels) do
		local arg = {
			type = 'execute',
			name = value,
			desc = "Delete this channel",
			func = function() InviteOMatic:DeleteChannel(key); end,
		};
		tinsert(self.options.args.channel.args.deleteChannel.args, arg);
	end
end

function InviteOMatic:UpdateIgnoreList()
	self.options.args.ignore.args.delete.args = {};
	for key,value in pairs(self.db.profile.ignoreList) do
		local arg = {
			type = 'execute',
			name = value,
			desc = "Delete this player",
			func = function() InviteOMatic:DeleteFromIgnore(key); end,
		};
		tinsert(self.options.args.ignore.args.delete.args, arg);
	end
end

function InviteOMatic:DeleteFromIgnore(key)
	local name = tostring(self.db.profile.ignoreList[key]);
	self:Debug("Removing "..name.." from ignore");
	tremove(self.db.profile.ignoreList, key);
	self:UpdateIgnoreList();
end

function InviteOMatic:DeleteInvitePattern(key)
	local name = tostring(self.db.profile.invitePatterns[key]);
	self:Debug("Removing "..name.." as inviteword");
	tremove(self.db.profile.invitePatterns, key);
	self:UpdatePatternsList();
end

function InviteOMatic:DeleteChannel(key)
	local name = tostring(self.db.profile.watchedChannels[key]);
	self:Debug("Removing "..name.." as watched channel");
	tremove(self.db.profile.watchedChannels, key);
	self:UpdateChannelList();
end

function InviteOMatic:WhisperEvent(msg, name)
	if( msg == nil ) then
		self:Debug("Got nil whisper");
		return;
	end

	self:Debug(name.." whispered for an invite");

	if( self:HasPattern(msg, self.db.profile.invitePatterns) ) then
		self:Invite(name);
	end
end

function InviteOMatic:ChannelEvent(msg, name, arg3, channel)
	if( msg == nil ) then
		self:Debug("Got nil whisper");
		return;
	end

	if( self:HasPattern(channel, self.db.profile.watchedChannels) ) then
		if( self:HasPattern(msg, self.db.profile.invitePatterns) ) then
			self:Invite(name);
		end
	end

	return;
end

function InviteOMatic:HasPattern(msg, patterns)
	local lower = string.lower(msg);
	for key,value in pairs(patterns) do
	
		local lowerV = string.lower(value);
	
		self:Debug("Msg: ["..lower.."], pattern: ["..lowerV.."]");
	
		if( string.find(lower, lowerV) ~= nil ) then
			return true;
		end
	end
	
	return false;
end

function InviteOMatic:Invite(name)
	if( not self:IsIgnored(name) ) then
		self:Debug("Inviting "..name.." to party");
		InviteUnit(name);
	else
		self:Debug(name.." is on your ignore list");
	end
end

function InviteOMatic:IsIgnored(name)
	local lower = string.lower(name);
	for key,value in pairs(self.db.profile.ignoreList) do
		local lowerV = string.lower(value);
		if( lowerV == lower ) then
			return true;
		end
	end
	
	return false;
end

function InviteOMatic:IsDebug()
	return self.db.profile.dbg;
end

function InviteOMatic:ToggleDebug()
	self.db.profile.dbg = not self.db.profile.dbg;
end

function InviteOMatic:OnInitialize()
    -- Called when the addon is loaded
	self:Print("InviteOMatic "..self.version.." loaded");
	self:UpdateIgnoreList();
	self:UpdatePatternsList();
	self:UpdateChannelList();
end

function InviteOMatic:OnEnable()
    -- Called when the addon is enabled
	self:Debug("IOM Enabled");
	self:RegisterEvent("CHAT_MSG_WHISPER", "WhisperEvent", false);
	self:RegisterEvent("CHAT_MSG_CHANNEL", "ChannelEvent", false);
end

function InviteOMatic:OnDisable()
    -- Called when the addon is disabled
	self:Debug("IOM Disabled");
end

function InviteOMatic:Debug(msg)
	if( self:IsDebug() ) then
		self:Print("Debug: "..msg);
	end
end